from Products.Five.browser import BrowserView


class RavenConnectionTest(ValueError):
    pass


class MakeError(BrowserView):

    def __call__(self):
        raise RavenConnectionTest(self.request.form)



import json
import os


def get_raven_config():
    return RavenConfig()


class RavenConfig(object):

    @property
    def enabled(self):
        return self.dsn is not None

    @property
    def dsn(self):
        return self._get_stripped_env_variable('RAVEN_DSN')

    @property
    def project_dist(self):
        return self._get_stripped_env_variable('RAVEN_PROJECT_DIST')

    @property
    def buildout_root(self):
        return self._get_stripped_env_variable('RAVEN_BUILDOUT_ROOT')

    @property
    def ignored_exception_classnames(self):
        ignored = set(['Redirect', 'NotFound', 'Unauthorized', 'Intercepted'])
        enabled = self._get_stripped_env_variable('RAVEN_ENABLE_EXCEPTIONS')
        if enabled:
            ignored -= set(map(str.strip, enabled.split(',')))

        return tuple(ignored)

    @property
    def tags(self):
        value = self._get_tags_from_file()
        value.update(self._get_tags_from_env())
        return value

    def _get_tags_from_env(self):
        value = self._get_stripped_env_variable('RAVEN_TAGS')
        if not value:
            return {}

        try:
            return json.loads(value)
        except ValueError:
            return {'RAVEN_TAGS': 'invalid configuration'}

    def _get_tags_from_file(self):
        value = self._get_stripped_env_variable('RAVEN_TAGS_FILE')
        if not value:
            return {}

        if not os.path.isfile(value):
            return {'RAVEN_TAGS_FILE': 'missing file'}

        try:
            with open(value) as tags_file:
                return json.load(tags_file)
        except ValueError:
            return {'RAVEN_TAGS_FILE': 'invalid content'}

    def _get_stripped_env_variable(self, name):
        value = os.environ.get(name, '')
        if value.strip():
            return value.strip()
        else:
            return None



from ftw.raven import client


class ClientMock(object):

    def __init__(self, dsn=None, raise_send_errors=False, transport=None,
                 install_sys_hook=True, **options):
        self.dsn = dsn
        self.captureException_calls = []

    def captureException(self, exc_info=None, **kwargs):
        kwargs.update({'exc_info': exc_info})
        self.captureException_calls.append(kwargs)

    @classmethod
    def install(klass):
        client.raven_client_class = klass


class CrashingClientMock(ClientMock):

    def __init__(self, *args, **kwargs):
        super(CrashingClientMock, self).__init__(*args, **kwargs)
        self.crashes = 0

    def captureException(self, exc_info=None, **kwargs):
        self.crashes += 1
        raise Exception('Not going to happen.')



from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import functional_session_factory
from ftw.builder.testing import set_builder_session_factory
from ftw.testing.layer import COMPONENT_REGISTRY_ISOLATION
from plone.app.testing import FunctionalTesting
from plone.app.testing import PloneSandboxLayer
from plone.memoize import forever
from plone.testing import z2
from zope.configuration import xmlconfig
import os


class RavenLayer(PloneSandboxLayer):
    defaultBases = (COMPONENT_REGISTRY_ISOLATION, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '  <include package="ftw.raven.demo" />'
            '</configure>',
            context=configurationContext)

        z2.installProduct(app, 'plone.app.linkintegrity')
        z2.installProduct(app, 'ftw.raven')

    def testTearDown(self):
        for name in os.environ.keys():
            if name.startswith('RAVEN'):
                del os.environ[name]

        # purge forever caches
        forever._memos.clear()


RAVEN_FIXTURE = RavenLayer()
RAVEN_FUNCTIONAL = FunctionalTesting(
    bases=(RAVEN_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="ftw.raven:functional")



from ftw.raven.client import get_raven_client
from ftw.raven.client import purge_raven_client
from ftw.raven.testing import RAVEN_FUNCTIONAL
from ftw.raven.tests.client_mock import ClientMock
from ftw.testbrowser import Browser
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest2 import TestCase
import transaction


class FunctionalTestCase(TestCase):
    layer = RAVEN_FUNCTIONAL

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.load_zcml_string = self.layer['load_zcml_string']
        ClientMock.install()

    def tearDown(self):
        purge_raven_client()

    def grant(self, *roles):
        setRoles(self.portal, TEST_USER_ID, roles)
        transaction.commit()

    def request_to_error_view(self, view='make_key_error', data=None,
                              login_as=None):
        # We need to make sure that the raven client mock is created
        # in this thread and not in the sub-thread created by the
        # testbrowser, in order to be able to read it later.
        get_raven_client()

        with Browser()(self.layer['app']) as browser:
            if login_as:
                browser.login(login_as)

            self.disable_handle_errors(browser)
            try:
                browser.open(view=view, data=data)
            except:
                pass

    def disable_handle_errors(self, browser):
        """If the testbrowser tells zope to not handle errors, as it does
        by default, the exception handling is not triggered, thus no errors
        are reported to sentry.
        In order to be able to test the reporting, we therefore need to
        enable error handling by removing the header.
        """
        header = ('X-zope-handle-errors', 'False')
        mechbrowser = browser.get_mechbrowser()
        if header in mechbrowser.addheaders:
            mechbrowser.addheaders.remove(header)



from ftw.raven.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from pkg_resources import get_distribution
import json
import os
import re


class TestJavaScript(FunctionalTestCase):

    @browsing
    def test_lib_headers(self, browser):
        os.environ['RAVEN_DSN'] = 'https://x:y@sentry.local/1'
        browser.login().open(view='ftw.raven.js')
        self.assertEquals('application/javascript; charset=utf-8',
                          browser.headers['Content-Type'])
        self.assertEquals('True',
                          browser.headers.get('X-Theme-Disabled'))

    @browsing
    def test_config_empty_when_not_configured(self, browser):
        browser.login().open(view='ftw.raven-config.js')
        self.assertEquals('', browser.contents)

    @browsing
    def test_user_context_configured(self, browser):
        os.environ['RAVEN_DSN'] = 'https://x:y@sentry.local/1'
        browser.login().open(view='ftw.raven-config.js')
        config = self.extract_config_from_javascript(browser.contents)
        self.assertEquals({'username': 'test-user',
                           'authentication': 'authenticated',
                           'fullname': '',
                           'ip_address': '',
                           'id': 'test_user_1_',
                           'email': ''},
                          config['user_context'])

    @browsing
    def test_dsn_configured(self, browser):
        os.environ['RAVEN_DSN'] = 'https://x:y@sentry.local/1'
        browser.login().open(view='ftw.raven-config.js')
        config = self.extract_config_from_javascript(browser.contents)
        self.assertEquals('https://x@sentry.local/1', config['dsn'])

    @browsing
    def test_server_name_in_config_args(self, browser):
        os.environ['RAVEN_DSN'] = 'https://x:y@sentry.local/1'
        browser.login().open(view='ftw.raven-config.js')
        config = self.extract_config_from_javascript(browser.contents)
        self.assertIn('serverName', config['options'])

    @browsing
    def test_release_in_config_args(self, browser):
        os.environ['RAVEN_DSN'] = 'https://x:y@sentry.local/1'
        os.environ['RAVEN_PROJECT_DIST'] = 'zope.interface'
        browser.login().open(view='ftw.raven-config.js')
        config = self.extract_config_from_javascript(browser.contents)
        version = get_distribution('zope.interface').version
        self.assertDictContainsSubset({'release': version}, config['options'])

    @browsing
    def test_tags_in_config_args(self, browser):
        os.environ['RAVEN_DSN'] = 'https://x:y@sentry.local/1'
        os.environ['RAVEN_TAGS'] = '{"deployment_type": "demo"}'
        browser.login().open(view='ftw.raven-config.js')
        config = self.extract_config_from_javascript(browser.contents)
        self.assertDictContainsSubset({'tags': {'deployment_type': 'demo'}},
                                      config['options'])

    def extract_config_from_javascript(self, js):
        js = re.sub('^var raven_config = ', '', js)
        js = re.sub(';$', '', js)
        return json.loads(js)



from ftw.raven import reporter
from ftw.raven.config import get_raven_config
from Products.Five.browser import BrowserView
import json
import os
import re
import socket


class RavenUserConfiguration(BrowserView):

    def __call__(self):
        if not get_raven_config().enabled:
            return ''

        config = {
            'dsn': self._get_dsn_without_private_key(),
            'options': self._get_install_args(),
            'user_context': reporter.prepare_user_infos(
                self.context, self.request,
                include_roles=False)}

        return 'var raven_config = {};'.format(
            json.dumps(config, sort_keys=True, indent=4))

    def _get_install_args(self):
        args = {}

        release = reporter.get_release()
        if release:
            args['release'] = release

        if hasattr(socket, 'gethostname'):
            args['serverName'] = socket.gethostname()

        tags = reporter.get_default_tags()
        if tags:
            args['tags'] = tags

        return args

    def _get_dsn_without_private_key(self):
        dsn = get_raven_config().dsn
        return re.sub(r'(://[^:]*):[^@]*(@)', '\g<1>\g<2>', dsn)


class RavenJavaScript(BrowserView):

    def __call__(self):
        self.request.response.setHeader('X-Theme-Disabled', 'True')
        self.request.response.setHeader(
            'Content-Type', 'application/javascript; charset=utf-8')
        return '\n\n'.join((self.get_resource_content('raven.min.js'),
                            self.get_resource_content('raven-integration.js')))

    def get_resource_content(self, filename):
        path = os.path.join(os.path.dirname(__file__),
                            'resources',
                            filename)
        with open(path, 'r') as shim:
            return shim.read()
